/* $Id: misc.c,v 1.1.1.1 1995/07/22 00:57:18 sverrehu Exp $ */
/**************************************************************************
 *
 *  FILE            misc.c
 *
 *  MODULE OF       motd
 *
 *  DESCRIPTION     
 *
 *
 *  WRITTEN BY      Sverre H. Huseby
 *
 *  MODIFICATIONS   
 *
 **************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>

#include "misc.h"

extern char *sys_errlist[];

/**************************************************************************
 *                                                                        *
 *                       P R I V A T E    D A T A                         *
 *                                                                        *
 **************************************************************************/

static char *Argv0 = NULL;      /* The name of the program. */



/**************************************************************************
 *                                                                        *
 *                    P U B L I C    F U N C T I O N S                    *
 *                                                                        *
 **************************************************************************/

void *xmalloc(size_t size)
{
    void *ret;
    
    if ((ret = malloc(size)) == NULL)
	Fatal("out of memory");
    return ret;
}

void *xrealloc(void *ptr, size_t size)
{
    void *ret;
    
    if ((ret = realloc(ptr, size)) == NULL)
	Fatal("out of memory");
    return ret;
}

/*-------------------------------------------------------------------------
 *
 *  NAME          SetProgName
 *
 *  FUNCTION      Specify the name of this program (from argv[0]).
 *
 *  INPUT         name   Name of this program, possibly including path.
 *
 *  RETURNS       Nothing
 *
 *  DESCRIPTION   The program name specified is used by the errorfunctions
 *                Error() and Fatal() to identify this program.
 *
 */
void SetProgName(const char *name)
{
    Argv0 = (char *) name;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          GetProgName
 *
 *  FUNCTION      Return the name of this program.
 *
 *  RETURNS       Pointer to the name of this program, without any path.
 *
 *  DESCRIPTION   The name depends on the string passed to SetProgName().
 *
 */
char *GetProgName(void)
{
    char *ret;

    #define PATHSEP '/'

    if (!Argv0)
        return("!!!");

    if ((ret = strrchr(Argv0, PATHSEP)) == NULL)
        ret = Argv0;
    else
        ++ret;

    return ret;
}



/*-------------------------------------------------------------------------
 *
 *  NAME          Fatal
 *
 *  FUNCTION      Show given message on stderr and abort the program.
 *
 *  SYNOPSIS      #include "message.h"
 *                void Fatal(const char *format, ...);
 *
 *  INPUT         format, ...
 *                        Arguments used as width printf().
 *
 *  RETURNS       Never returns. The program is aborted.
 *
 *  DESCRIPTION   Prints the name of this program followed by ": " and the
 *                given message.
 *
 */
void Fatal(const char *format, ...)
{
    va_list ap;

    fprintf(stderr, "%s: ", GetProgName());
    va_start(ap, format);
    vfprintf(stderr, format, ap);
    va_end(ap);
    exit(99);
}



/*-------------------------------------------------------------------------
 *
 *  NAME          FatalPerror
 *
 *  FUNCTION      Show an errno-dependant error message on stderr,
 *                and abort the program.
 *
 *  SYNOPSIS      #include "message.h"
 *                void FatalPerror(const char *text);
 *
 *  INPUT         text    Text to display before the message. If NULL,
 *                        the name of this program is used.
 *
 *  RETURNS       Nothing, never returns.
 *
 *  DESCRIPTION   Prints the given text followed by ": " and the
 *                correct errormessage, and aborts the program.
 *
 *                If errno indicates "no error", nothing is printed,
 *                and the program keeps running.
 *
 */
void FatalPerror(const char *text)
{
    if (!errno)
        return;
    fprintf(stderr, "%s: %s\n",
            text ? text : GetProgName(),
            sys_errlist[errno]);
    exit(99);
}
